matlabpool open local;

% Load the data
load MCDataS500R500; N = SampleSize;

% Set the seed for this simulation
seed = 8; rand('state',8); randn('state',8);

% In this file we will cycle through lambda constraints
pen = [0, N^(1/3), N^(1/2), N^(2/3)];

% Create the Y we will need
Y = 2*sin(pi*X) + E;

% Set the parameters we will run this under
L = 0;                 % The constraint value         
B = [10^4; 10^4];      % The smoothness constraint, first on beta, then on bootstrap
VIT = -0.8:0.4:0.8;    % Create a grid for the Ts
k_n = 1;               % The number of knots for Bsplines
Ord_X = 3;             % The order of the Bsplines

% And the structures we'll save this in.
quant = NaN(Replications,length(pen)); 
opt = NaN(Replications,1); 
xflag = NaN(Replications,1);

% Find out how many bases we actually have
T = 0:(k_n+1); T = augknt(T,Ord_X); 
T = spcol(T, Ord_X, 1:10); nbasis = size(T,2);
Save_opt = zeros(size(T,2), Replications);
clear T;

% Moving on to the smoothness constraint, we need to set up a grid of
% points to evaluate the derivative of the splines
div4g = 0.01; knots = 0:1/(k_n+1):1; knots = norminv(knots(2:end-1));
knots = 2*(normcdf(knots/3) - 0.5); knots = [-1 knots 1]; knots = augknt(knots,Ord_X);
T1 = (-1:div4g:1)'; T1 = repmat(T1,Ord_X,1); T1 = sort(T1);
T2 = spcol(knots, Ord_X, T1);

XGrid = ones(length(-1:div4g:1), nbasis, Ord_X);
for i = 1:Ord_X
    for j = 1:length(-1:div4g:1)
        XGrid(j,:,i) = T2(Ord_X*(j-1) + i,:);
    end
end

% Now with this grid we'll do numerical integration
LM = zeros(size(XGrid,2));
for d = 1:size(XGrid,3)
    for j = 1:size(XGrid,2)
        T = repmat(XGrid(:,j,d), 1, size(XGrid,2)).*XGrid(:,:,d);
        LM(j,:) = LM(j,:) + trapz(T)*div4g;
    end
end

clear T T1 T2 i j d div4g;

% Set the starting values of the first optimization
x0 = zeros(nbasis,1);

% And now iterate through each replication
for k = 1:Replications
tic
    % Evaluate the X at the different splines. Knots put at the percentiles
    XSplines = spcol(knots, Ord_X, X(:,k));
    
    % And put the data together
    Data = [Y(:,k) Z(:,k) XSplines];
    
    % Finish setting up the constraints
    Am = []; Bm = []; LB = []; UB = [];
    
    % This is the value of the function at the point 0
    Aeq = spcol(knots, Ord_X, 0); 
    BootAeq = [Aeq zeros(1,size(XSplines,2)); zeros(1,size(XSplines,2)) Aeq];
    
    % And optimization options - for full sample problem
    options = optimset('fmincon'); options.GradObj = 'on'; options.Display = 'off';
    options.GradConstr = 'on'; options.DerivativeCheck = 'off'; 
    options.MaxFunEvals = 10000; options.MaxIter = 10000; 
    options.LargeScale = 'off'; options.Algorithm = 'active-set';
    options.TolFun = 1e-8;
    
    % Now look at the bounds
    Beq = L; BootBeq = [L; 0];
    
    % Set up the functions to be called
    fobj = @(x)f_bier_minmax(x,Data,VIT);
    fcon = @(x)f_bier_nonlcon(x,LM,B(1));
    
    % And finally call the optimizer
    [Save_opt(:,k) Tempsol Temp xflag(k)] = fminimax(fobj, x0, Am, Bm, Aeq, Beq, LB, UB, fcon, options);
    opt(k) = max(Tempsol);            
    
    % Calculate the bootstrap statistic. Get sample, etc.
    R = 500; BI = int64(1 + rand(N,R)*(N-1));
    BVal = zeros(R,1); BXflag = zeros(R,1);
    
    % Now get the bootstrap critical value - but only if you converged
    if xflag(k) > 0
        
        % Set the starting value for the bootstrap
        x0 = zeros(size(XSplines,2)*2,1);

        % Evaluate this at different penalty choices
        for p = 1:length(pen)
            BCon = pen(p);
            
            parfor b = 1:R
                %Put together the bootstrap sample
                BY = Y(BI(:,b),k); BZ = Z(BI(:,b),k);
            
                % And the Xsplines
                BXS = XSplines*0;
                for c = 1:size(XSplines,2)
                    T = XSplines(:,c);
                    BXS(:,c) = T(BI(:,b));
                end            
            
                BData = [BY BZ BXS; Data];
            
                % St up the functions we will call
                fbootobj = @(x)f_boot_minmax(x,BData,VIT,BCon);
                fbootcon = @(x)f_boot_nonlcon(x,LM,B);
            
                % Call the optimizer
                [T Temp BXflag(b)] = fminimax(fbootobj, x0, Am, Bm, BootAeq, BootBeq, LB, UB, fbootcon, options);           
           
                BVal(b) = max(Temp);           
            end
            
            % Get the quantile we are looking for
            BVal = BVal(BXflag > 0);
            quant(k,p) = sum(BVal <= opt(k))/length(BVal);                
        end
                               
        k
    else                
        quant(k,:) = NaN;
    end
toc    
    % And use the optimum as the starting point for the next optimization
    x0 = Save_opt(:,k);
end

save MCOutput quant opt pen seed;

clear;